package com.ruoyi.system.service.impl;

import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.system.listener.ExcelListener;
import com.ruoyi.system.mapper.WagesMapper;
import com.ruoyi.system.mapper.WagesSalaryProofMapper;
import com.ruoyi.system.model.wages.dao.*;
import com.ruoyi.system.model.wages.vo.WagesOverviewOriginalDataVO;
import com.ruoyi.system.service.WagesService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 2024/4/27
 */
@Service
public class WagesServiceImpl implements WagesService {
    private Logger logger = LoggerFactory.getLogger(WagesServiceImpl.class);
    @Autowired
    WagesMapper wagesMapper;
    @Autowired
    WagesSalaryProofMapper wagesSalaryProofMapper;
    @Transactional
    @Override
    public String readOriginalData(MultipartFile file, String date) {
        try {
//            if (Test.put("工资导入", "1")){
//                return "不可重复点击";
//            }
            wagesMapper.deleteOverviewToDate(date);
            wagesMapper.deleteDetailsToDate(date);
            InputStream inputStream1 = file.getInputStream();
            List<WagesDetailsOriginalExcelSheet1DAO> wagesDetSheet1List = EasyExcelFactory.read(inputStream1, WagesDetailsOriginalExcelSheet1DAO.class, new ExcelListener<WagesDetailsOriginalExcelSheet1DAO>()).sheet(0).headRowNumber(1).doReadSync();
            inputStream1.close();
            InputStream inputStream2 = file.getInputStream();
            List<WagesDetailsOriginalExcelSheet2DAO> wagesDetSheet2List = EasyExcelFactory.read(inputStream2, WagesDetailsOriginalExcelSheet2DAO.class, new ExcelListener<WagesDetailsOriginalExcelSheet2DAO>()).sheet(1).headRowNumber(1).doReadSync();
            inputStream2.close();
            InputStream inputStream = file.getInputStream();
            List<WagesOverviewOriginalExcelDAO> wagesOveList = EasyExcelFactory.read(inputStream, WagesOverviewOriginalExcelDAO.class, new ExcelListener<WagesOverviewOriginalExcelDAO>()).sheet(2).headRowNumber(1).doReadSync();
            inputStream.close();
            List<WagesDetailsOriginalExcelDAO> wagesDetList = new ArrayList<>();
            int size = wagesDetSheet1List.size();
            for (int i = 0; i < size; i++) {
                WagesDetailsOriginalExcelSheet1DAO wagesSheet1 = wagesDetSheet1List.get(i);
                WagesDetailsOriginalExcelSheet2DAO wagesSheet2 = wagesDetSheet2List.stream().filter(v -> v.getUnitName().equals(wagesSheet1.getUnitName()))
                        .collect(Collectors.toList()).get(0);
                WagesDetailsOriginalExcelDAO wages = new WagesDetailsOriginalExcelDAO(wagesSheet1.getYear(), wagesSheet1.getMonth(), wagesSheet1.getUnitName(), wagesSheet1.getPeopleCount(), wagesSheet1.getWagesPayable(), wagesSheet1.getHouseRent(), wagesSheet1.getAccommodation(), wagesSheet1.getHeatingFee(), wagesSheet1.getTradeUnionDues(), wagesSheet1.getCompensationDeduction(),
                        wagesSheet1.getOtherDeductions(), wagesSheet1.getMutualMedicalAid(),
                        wagesSheet1.getCommunicationFee(), wagesSheet1.getTax1(), wagesSheet1.getTax2(), wagesSheet1.getPensionInsurance(), wagesSheet1.getUnemploymentInsurance(), wagesSheet1.getMedicalInsurance(), wagesSheet1.getAnnuity(), wagesSheet1.getHousingProvidentFund(),
                        wagesSheet1.getEnterpriseFund(), wagesSheet1.getAnnuityIncomeTax(),
                        wagesSheet2.getTotalWithholding(), wagesSheet2.getNetSalary(),
                        wagesSheet2.getOneChildFee(), wagesSheet2.getMedicalSubsidy(), wagesSheet2.getHealthCareFee(), wagesSheet2.getClassMealFee(), wagesSheet2.getSoapFee(), wagesSheet2.getNursingFee(), wagesSheet2.getHeatstrokePreventionSubsidy(), wagesSheet2.getHeatingSubsidy(), wagesSheet2.getManuscriptFee(), wagesSheet2.getDriverAllowance(), wagesSheet2.getGraduateSettlementFee(), wagesSheet2.getOneTimeDisabilitySubsidy(), wagesSheet2.getLectureAndExamFee(), wagesSheet2.getOtherSubsidies(), wagesSheet2.getTotalPayment(), wagesSheet2.getTotalSalaryDisbursement());
                wagesDetList.add(wages);
            }
            if (wagesDetList.size() > 0){
                wagesMapper.deleteDetailsOriginal(wagesDetList.get(0).getMonth(), wagesDetList.get(0).getYear());
            }
            if (wagesOveList.size() > 0) {
                wagesMapper.deleteWagesOverviewOriginal(wagesOveList.get(0).getMonth(), wagesOveList.get(0).getYear());
            }
            wagesMapper.insertDetailsOriginal(wagesDetList, date);
            wagesMapper.insertWagesOverviewOriginal(wagesOveList, date);
            return "导入成功！";
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public List<WagesOverviewOriginalDataVO> getSalaryList() {
        logger.info("getSalaryList");
        List<WagesOverviewOriginalDataVO> salaries = wagesMapper.queryList();
        return salaries;
    }

    @Override
    public List<WagesDetailsOriginalExcelDAO> details() {
        return wagesMapper.detailsSelect();
    }

    @Override
    public JSONObject selectWagesVoucherList(String ss) {
        JSONObject object = new JSONObject();
        //List<WagesSalaryProof>  borrowlists = new ArrayList<>();
        JSONArray borrowlists = new JSONArray();
        //List<WagesSalaryProof>  loanlists = new ArrayList<>();
        JSONArray  loanlists = new JSONArray();
        JSONObject jsonObject = JSON.parseObject(ss);
        JSONArray borrow = jsonObject.getJSONArray("borrow");
        JSONArray loan = jsonObject.getJSONArray("loan");
        for (int i = 0; i < borrow.size(); i++) {
            JSONObject jsonObject1 = borrow.getJSONObject(i);
            String abStract = jsonObject1.getString("classary");
            String sourceOfExpenses = jsonObject1.getString("name");
            String isDebit = jsonObject1.getString("is_debit");

            WagesSalaryProof wagesSalaryProof = new WagesSalaryProof();
            wagesSalaryProof.setAbStract(abStract);
            wagesSalaryProof.setSourceOfExpenses(sourceOfExpenses);
            wagesSalaryProof.setIsDebit(isDebit);
            WagesSalaryProof wagesSalaryProof1 = wagesSalaryProofMapper.selectWagesVoucherList(wagesSalaryProof);

//            wagesSalaryProof.setAccountCode(wagesSalaryProof1.getAccountCode());
//            wagesSalaryProof.setAuxiliaryItemA(wagesSalaryProof1.getAuxiliaryItemA());
//            wagesSalaryProof.setAuxiliaryItemB(wagesSalaryProof1.getAuxiliaryItemB());
            //borrowlists.add(wagesSalaryProof);
            if(wagesSalaryProof1 != null){
                System.err.println("wagesSalaryProof1====" + wagesSalaryProof1.toString() + i);
                jsonObject1.put("account_code",wagesSalaryProof1.getAccountCode());
                jsonObject1.put("auxiliary_item_a",wagesSalaryProof1.getAuxiliaryItemA());
                jsonObject1.put("auxiliary_item_b",wagesSalaryProof1.getAuxiliaryItemB());
            }else{
                 //如果项目带前带（，去获取相似项目的值
//                 if(sourceOfExpenses.startsWith("(")){
//                     for (int i = 0; i < borrowlists.size(); i++) {
//                         JSONObject obj = borrowlists.getJSONObject(i);
//                         if (obj.getString("classary").equals(abStract) && obj.getInt("age") == searchAge) {
//                             found = obj;
//                             break;
//                         }
//                     }
//
//                 }
            }

            borrowlists.add(jsonObject1);
        }
        for (int i = 0; i < loan.size(); i++) {
            JSONObject jsonObject2 = loan.getJSONObject(i);
            String abStract = jsonObject2.getString("classary");
            String sourceOfExpenses = jsonObject2.getString("name");
            String isDebit = jsonObject2.getString("is_debit");
            WagesSalaryProof wagesSalaryProof = new WagesSalaryProof();
            wagesSalaryProof.setAbStract(abStract);
            wagesSalaryProof.setSourceOfExpenses(sourceOfExpenses);
            wagesSalaryProof.setIsDebit(isDebit);
            WagesSalaryProof wagesSalaryProof2 = wagesSalaryProofMapper.selectWagesVoucherList(wagesSalaryProof);

//            wagesSalaryProof.setAccountCode(wagesSalaryProof1.getAccountCode());
//            wagesSalaryProof.setAuxiliaryItemA(wagesSalaryProof1.getAuxiliaryItemA());
//            wagesSalaryProof.setAuxiliaryItemB(wagesSalaryProof1.getAuxiliaryItemB());
//            loanlists.add(wagesSalaryProof);
            if(wagesSalaryProof2 != null){
                System.err.println("wagesSalaryProof2====" + wagesSalaryProof2.toString() + i);
                jsonObject2.put("account_code",wagesSalaryProof2.getAccountCode());
                jsonObject2.put("auxiliary_item_a",wagesSalaryProof2.getAuxiliaryItemA());
                jsonObject2.put("auxiliary_item_b",wagesSalaryProof2.getAuxiliaryItemB());
            }

            loanlists.add(jsonObject2);
        }

//        com.alibaba.fastjson2.JSONArray borrowjson = com.alibaba.fastjson2.JSONArray.of(borrowlists);
//        com.alibaba.fastjson2.JSONArray loanjson = com.alibaba.fastjson2.JSONArray.of(loanlists);
        object.put("borrow",borrowlists);
        object.put("loan",loanlists);
        return object;
    }
}
